@@ -28,5 +28,4 @@ class BenchLinesSerialChunk(BenchBase):
             self.x, self.y, self.z, name=name, line_type=line_type,
             corner_mask=corner_mask_to_bool(corner_mask), total_chunk_count=total_chunk_count,
         )
-        for level in self.levels:
-            cont_gen.lines(level)
+        cont_gen.multi_lines(self.levels)
