@@ -38,5 +38,4 @@ class BenchLinesThreaded(BenchBase):
             corner_mask=corner_mask_to_bool(corner_mask), total_chunk_count=total_chunk_count,
             thread_count=thread_count,
         )
-        for level in self.levels:
-            cont_gen.lines(level)
+        cont_gen.multi_lines(self.levels)
