@@ -4,6 +4,8 @@
 #include <pybind11/pybind11.h>
 #include <pybind11/numpy.h>
 
+namespace contourpy {
+
 namespace py = pybind11;
 
 // quad/point index type, the same as for numpy array shape/indices (== npy_intp).  Also used for
@@ -19,10 +21,13 @@ typedef uint32_t offset_t;
 // Input numpy array classes.
 typedef py::array_t<double, py::array::c_style | py::array::forcecast> CoordinateArray;
 typedef py::array_t<bool,   py::array::c_style | py::array::forcecast> MaskArray;
+typedef py::array_t<double> LevelArray;  // Doesn't have to be contiguous.
 
 // Output numpy array classes.
 typedef py::array_t<double>   PointArray;
 typedef py::array_t<uint8_t>  CodeArray;
 typedef py::array_t<offset_t> OffsetArray;
 
+} // namespace contourpy
+
 #endif // CONTOURPY_COMMON_H
