@@ -3,6 +3,8 @@
 #ifndef MPL_KIND_CODE
 #define MPL_KIND_CODE
 
+namespace contourpy {
+
 typedef enum
 {
     MOVETO = 1,
@@ -10,4 +12,6 @@ typedef enum
     CLOSEPOLY = 79
 } MplKindCode;
 
+} // namespace contourpy
+
 #endif // MPL_KIND_CODE
