@@ -3,6 +3,8 @@
 
 #include <iosfwd>
 
+namespace contourpy {
+
 typedef enum
 {
     Outer,
@@ -11,4 +13,6 @@ typedef enum
 
 std::ostream &operator<<(std::ostream &os, const OuterOrHole& outer_or_hole);
 
+} // namespace contourpy
+
 #endif // CONTOURPY_OUTER_OR_HOLE_H
