@@ -3,6 +3,8 @@
 
 #include "base.h"
 
+namespace contourpy {
+
 class SerialContourGenerator : public BaseContourGenerator<SerialContourGenerator>
 {
 public:
@@ -22,12 +24,17 @@ private:
     public:
         explicit Lock(SerialContourGenerator& contour_generator)
         {}
-
-        void unlock()
-        {}
     };
 
+    // Write points and offsets/codes to output numpy arrays.
+    void export_filled(const ChunkLocal& local, std::vector<py::list>& return_lists);
+
+    // Write points and offsets/codes to output numpy arrays.
+    void export_lines(const ChunkLocal& local, std::vector<py::list>& return_lists);
+
     void march(std::vector<py::list>& return_lists);
 };
 
+} // namespace contourpy
+
 #endif // CONTOURPY_SERIAL_H
